@@ -1,11 +1,14 @@
 package PGObject::Simple::Role;
 
-use 5.006;
-use strict;
-use warnings;
+use 5.010;
+
+use Carp::Clan qr/^PGObject\b/;
+use Log::Any qw($log);
+
+use PGObject::Simple ':full', '!dbh';
+
 use Moo::Role;
-use PGObject::Simple;
-use Carp;
+# Anything imported after `use Moo::Role;` will be composed into consuming packages.
 
 =head1 NAME
 
@@ -13,20 +16,19 @@ PGObject::Simple::Role - Moo/Moose mappers for minimalist PGObject framework
 
 =head1 VERSION
 
-Version 1.01
+Version 2.1.0
 
 =cut
 
-our $VERSION = '1.01';
-
+our $VERSION = '2.1.0';
 
 =head1 SYNOPSIS
 
-Take the following (Moose) class:
+Take the following (Moo) class:
 
     package MyAPP::Foo;
     use PGObject::Util::DBMethod;
-    use Moose;
+    use Moo;
     with 'PGObject::Simple::Role';
 
     has id  => (is => 'ro', isa => 'Int', required => 0);
@@ -34,7 +36,7 @@ Take the following (Moose) class:
     has bar => (is => 'ro', isa => 'Str', required => 0);
     has baz => (is => 'ro', isa => 'Int', required => 0);
 
-    sub get_dbh {
+    sub _get_dbh {
         return DBI->connect('dbi:Pg:dbname=foobar');
     }
     #  PGObject::Util::DBMethod exports this
@@ -68,27 +70,49 @@ declarative mapping.
 
 =head1 ATTRIBUTES AND LAZY GETTERS
 
+
 =cut
 
-# Private attribute for database handle, not intended to be directly set.
 
-has _DBH => ( 
-       is => 'lazy', 
-       isa => sub { 
-                    croak "Expected a database handle.  Got $_[0] instead"
-                       unless eval {$_[0]->isa('DBI::db')};
+has _dbh => (  # use dbh() to get and set_dbh() to set
+       is => 'lazy',
+       isa => sub {
+           croak $log->error( "Expected a database handle.  Got $_[0] instead" )
+               unless eval {$_[0]->isa('DBI::db')};
        },
 );
 
-sub _build__DBH {
+has _DBH => ( # backwards compatible for 1.x.
+        is => 'lazy',
+       isa => sub {
+           warn $log->warn( 'deprecated _DBH used.  rename to _dbh when you can' );
+           croak $log->error( "Expected a database handle.  Got $_[0] instead" )
+               unless eval {$_[0]->isa('DBI::db')};
+       },
+);
+
+sub _build__dbh {
     my ($self) = @_;
+    return $self->{_DBH} if $self->{_DBH};
     return $self->_get_dbh;
 }
 
-has _Registry => (is => 'lazy');
+sub _build__DBH {
+    my ($self) = @_;
+    return $self->{_dbh} if $self->{_dbh};
+    return $self->_dbh;
+}
 
-sub _build__Registry {
-    return _get_registry();
+sub _get_dbh {
+    croak 'Invoked _get_dbh from role improperly.  Subclasses MUST set this method';
+}
+
+has _registry => (is => 'lazy');
+
+sub _build__registry {
+    my ($self) = @_;
+    return $self->_get_registry() if $self->can('_get_registry');
+    _get_registry();
 }
 
 =head2 _get_registry
@@ -128,131 +152,55 @@ stored prcedures to an object class.
 =cut
 
 sub _build__funcprefix {
-    return $_[0]->_get_prefix;
+    my ($self)  = @_;
+    return $self->_get_prefix;
 }
 
 sub _get_prefix {
     return '';
 }
 
-has _PGObject_Simple => (
-    is => 'lazy',
-);
+=head1 READ ONLY ACCESSORS (PUBLIC)
 
-sub _build__PGObject_Simple {
+=head2 dbh
+
+Wraps the PGObject::Simple method
+
+=cut
+
+sub dbh {
     my ($self) = @_;
-    return PGObject::Simple->new() unless ref $self;
-    $self->_DBH;
-    $self->_funcprefix;
-    my $obj = PGObject::Simple->new(%$self);
-    $obj->_set_registry($self->_registry);
-    return $obj;
+    if (ref $self){
+        return $self->_dbh;
+    }
+    return "$self"->_get_dbh;
 }
 
-has _registry => ( is => 'lazy' );
+=head2 funcschema
 
-sub _build__registry {
-    return _get_registry();
-}
-
-=head2 _get_dbh
-
-Subclasses or sub-roles MUST implement a function which returns a DBI database
-handle (DBD::Pg 2.0 or hgher required).  If this is not overridden an exception
-will be raised.
+Returns the schema bound to the object
 
 =cut
 
-sub _get_dbh {
-    croak 'Subclasses MUST set their own get_dbh methods!';
+sub funcschema {
+    my ($self)  = @_;
+    return $self->_funcschema if ref $self;
+    return "$self"->_get_schema();
 }
 
-=head2 call_procedure
+=head2 funcprefix
 
-Identical interface to PGObject::Simple->call_procedure.
-
-This can be used on objects or on the packages themselves.  I.e.  
-mypackage->call_procedure() and $myobject->call_procedure() both work.
+Prefix for functions
 
 =cut
 
-sub call_procedure {
-    my $self = shift @_;
-    my %args = @_;
-    my $obj = _build__PGObject_Simple($self);
-    $obj->{_DBH} = $self->_DBH if ref $self and !$args{dbh};
-    $obj->{_DBH} = "$self"->_get_dbh unless ref $self or $args{dbh};
-    if (ref $self){
-        $args{funcprefix} = $self->_funcprefix 
-                  unless defined $args{funcprefix} or !ref $self;
-        $args{funcschema} = $self->_funcschema 
-                  unless defined $args{funcschema} or !ref $self;
-    } else {
-        $args{funcprefix} = "$self"->_get_prefix
-                 unless defined $args{funcprefix} or ref $self;
-        $args{funcschema} = "$self"->_get_schema
-                 unless defined $args{funcschema} or ref $self;
-    }
-    my @rows = $obj->call_procedure(%args);
-    return @rows if wantarray;
-    return shift @rows;
+sub funcprefix {
+    my ($self) = @_;
+    
+    return $self->_funcprefix if ref $self;
+    return "$self"->_get_prefix();
 }
 
-=head2 call_dbmethod
-
-Identical interface to PGObject::Simple->call_dbmethod
-
-This can be used on objects or on the packages themselves.  I.e.  
-mypackage->call_dbmethod() and $myobject->call_dbmethod() both work.
-
-=cut
-
-sub call_dbmethod {
-    my $self = shift @_;
-    my %args = @_;
-    croak 'No function name provided' unless $args{funcname};
-
-    $args{dbh} = $self->_DBH if ref $self and !$args{dbh};
-    $args{dbh} = "$self"->_get_dbh() unless $args{dbh};
-    if (ref $self){
-        $args{funcprefix} = $self->_funcprefix unless defined $args{funcprefix};
-        $args{funcschema} = $self->_funcschema unless $args{funcschema};
-    } else {
-        $args{funcprefix} = "$self"->_get_prefix 
-             unless defined $args{funcprefix};
-        $args{funcschema} = "$self"->_get_schema unless $args{funcschema};
-    }
-    $args{funcprefix} ||= '';
-
-    my $info = PGObject->function_info(%args);
-
-    my $dbargs = [];
-    for my $arg (@{$info->{args}}){
-        $arg->{name} =~ s/^in_//;
-        my $db_arg;
-        eval { $db_arg = $self->can($arg->{name})->($self) } if ref $self;
-        if ($args{args}->{$arg->{name}}){
-            $db_arg = $args{args}->{$arg->{name}};
-        }
-        if (eval {$db_arg->can('to_db')}){
-           $db_arg = $db_arg->to_db;
-        }
-        if ($arg->{type} eq 'bytea'){
-           $db_arg = { type => 'bytea', value => $db_arg};
-        }
-        push @$dbargs, $db_arg;
-    }
-    $args{args} = $dbargs;
-    my @rows;
-    if (ref $self){
-        @rows = $self->call_procedure(%args);
-    } else {
-        @rows = "$self"->call_procedure(%args);
-    }
-    return @rows if wantarray;
-    return shift @rows;
-}    
-
 =head1 REMOVED METHODS
 
 These methods were once part of this package but have been removed due to
@@ -280,6 +228,12 @@ Chris Travers,, C<< <chris.travers at gmail.com> >>
 
 =head1 BUGS
 
+Please report any bugs or feature requests to C<bug-pgobject-simple-role at rt.cpan.org>, or through
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=PGObject-Simple-Role>.  I will be notified, and then you'll
+Chris Travers,, C<< <chris.travers at gmail.com> >>
+
+=head1 BUGS
+
 Please report any bugs or feature requests to C<bug-pgobject-simple-role at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=PGObject-Simple-Role>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
@@ -300,19 +254,11 @@ You can also look for information at:
 
 =item * RT: CPAN's request tracker (report bugs here)
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=PGObject-Simple-Role>
+L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=PGObject-Simple-Role>
 
-=item * AnnoCPAN: Annotated CPAN documentation
+=item * MetaCPAN
 
-L<http://annocpan.org/dist/PGObject-Simple-Role>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/PGObject-Simple-Role>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/PGObject-Simple-Role/>
+L<https://metacpan.org/dist/PGObject-Simple-Role>
 
 =back
 
@@ -322,7 +268,8 @@ L<http://search.cpan.org/dist/PGObject-Simple-Role/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2013 Chris Travers,.
+  Copyright 2013-2017 Chris Travers
+  Copyright 2016-2021 The LedgerSMB Core team
 
 Redistribution and use in source and compiled forms with or without 
 modification, are permitted provided that the following conditions are met:
